package suicideChess;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;

import suicideChess.Move.NotAValidMoveException;
import suicideChess.Square.NotAValidSquare;

/**
 * Main file (the game in itself)
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$
 */

public class SuicideChess {

	/*
	 * Those flags are used to perform extra checks during the debugging of the
	 * program. They may be safely all set to false once the program is stable.
	 * It should improve performance a lot.
	 */
	/**
	 * does BitBoard.class removePiece function checks if removing piece is legal ?
	 */ 
	public static final boolean BITBOARD_REMOVEPIECE_CHECK_REMOVE = true;
	/**
	 * does Square.class checks if the strings are valid (is "z9" a valid square ? 
	 */
	public static final boolean SQUARE_CHECK_INVALID = true;

	private static final int MAIN_VERSION_NUMBER = 0;
	private static final int REVISION_NUMBER = 17;

    
	/**
	 * The main function
	 * @param args No parameters should be transmitted to this function.
	 */
	public static void main(String[] args) {
		System.out.println("   Welcome to SuicideChess v"+MAIN_VERSION_NUMBER+"."+REVISION_NUMBER+"!");
		System.out.println();
        BufferedReader moveInput = new BufferedReader(new InputStreamReader(System.in));
		
		try {
			Board bitboard = new Board();
			bitboard.display();
			
			/*int playerColor = Piece.WHITE;
			System.out.println("White: ");*/
			
			
			while (true) {
				try {
					String whatMove= moveInput.readLine();

					if (whatMove.startsWith("quit")) {
						System.out.println("Goodbye!");
						break;
					}
					
					if (whatMove.startsWith("hint")) {
						Rules rules = new Rules();
						ArrayList<Move> allLegalMoves =
								rules.legalMovesFromSquare(new Square(whatMove.substring(5,7)),bitboard);
						for(int i = 0; i<allLegalMoves.size(); i++) {
							allLegalMoves.get(i).display();
						}
						continue;
					}
                    
                    if (whatMove.startsWith("force")) {
                        Move theMove = new Move(whatMove.substring(6,10), bitboard);
                        theMove.display();
                        bitboard.doMove(theMove);
                        bitboard.display();
                        continue;
                    }
					
					Move theMove = new Move(whatMove, bitboard);
					Rules rules = new Rules();
					ArrayList<Move> allLegalMoves =
					    rules.legalMovesFromSquare(new Square(whatMove.substring(0,2)),bitboard);
					int foundMoveIndex = -1;
					for (int moveIndex = 0; moveIndex < allLegalMoves.size(); moveIndex++) {
                        if (allLegalMoves.get(moveIndex).toSquare().isEqual(theMove.toSquare())) {
					        foundMoveIndex=moveIndex;
					        break;
					    }
					}
					if (foundMoveIndex == -1) {
					    System.out.println("This move is not valid. Please type 'hint "+whatMove.substring(0,2)
					            +"' to see available moves.");
					} else {
					    allLegalMoves.get(foundMoveIndex).display();
					    bitboard.doMove(allLegalMoves.get(foundMoveIndex));
					    bitboard.display();               
					}					    
					                    
					/*if (playerColor == Piece.WHITE) {
						playerColor = Piece.BLACK;
						System.out.println("Black: ");
					} else {
						playerColor = Piece.WHITE;
						System.out.println("White: ");
					}*/
				
					
				} catch (NotAValidMoveException err) {
					System.out.println(err);
					continue;
		        } catch (NotAValidSquare err) {
		        	System.out.println(err);
		        	continue;
		        } catch (Exception err) {
		        	System.out.println(err);
		        	break;
		        }
			}
		} catch (NotAValidSquare e) {
			e.printStackTrace();
		}
	}

}
